###################################################
#   This code is based on EE367 HW6, task 2
#   (Gordon Wetzstein, 10/2021)
###################################################

# import packages
import numpy as np
from numpy.fft import fft2, ifft2

import skimage.io as io
from skimage.metrics import mean_squared_error, peak_signal_noise_ratio, structural_similarity
from skimage.color import rgb2gray, rgb2ycbcr, ycbcr2rgb
from skimage.filters import gaussian
from copy import deepcopy

from pypher.pypher import psf2otf
from pathlib import Path
from tqdm import tqdm
from itertools import product
import matplotlib.pyplot as plt

# HQS+TV
from removal_hqs_tv import *


# select target image and load it
name = '90-1'
img = io.imread(f'data/A/{name}.png').astype(float)/255
mask = io.imread(f'data/B/{name}.png').astype(float)/255
img_gt = io.imread(f'data/C/{name}.png').astype(float)/255
    
b_anisotropic = True

# number of iterations for the solver
num_iters = 75

# HQS+TV solver parameters
rho         = 0.08  # rho parameter of HQS
lam         = 0.15     # relative weight of TV term

ycbcr = rgb2ycbcr(deepcopy(img))
ycbcr[..., 0] = removal_hqs_tv(ycbcr[..., 0],mask,lam,rho,num_iters,b_anisotropic)
rgb = ycbcr2rgb(ycbcr)
rgb = np.clip(rgb, 0, 1)
io.imsave(f'results/{name}-removal.png', rgb)
MSE = mean_squared_error(img_gt, rgb)
PSNR = peak_signal_noise_ratio(img_gt, rgb)
SSIM = structural_similarity(img_gt, rgb, channel_axis=2)

RMSE = np.sqrt(MSE)
print(f"RMSE: {RMSE}")
print(f"PSNR: {PSNR}")
print(f"SSIM: {SSIM}")



# mask = io.imread(f'data/B/{name}.png')
# show results
# fig = plt.figure()

# ax = fig.add_subplot(2, 2, 1)
# ax.imshow(img)
# ax.set_title("Shadowed Image", fontsize=10)
# ax.xaxis.set_visible(False)
# ax.yaxis.set_visible(False)

# # ax = fig.add_subplot(2, 2, 2)
# # ax.imshow(mask, cmap='gray')
# # ax.set_title("Shadow Mask", fontsize=10)
# # ax.xaxis.set_visible(False)
# # ax.yaxis.set_visible(False)

# ax = fig.add_subplot(2, 2, 2)
# ax.imshow(img_gt)
# ax.set_title("Shadow-free Image", fontsize=10)
# ax.xaxis.set_visible(False)
# ax.yaxis.set_visible(False)

# ax = fig.add_subplot(2, 2, 3)
# ax.imshow(rgb1)
# ax.set_title(r"$\rho$=0.08", fontsize=10)
# ax.xaxis.set_visible(False)
# ax.yaxis.set_visible(False)

# ax = fig.add_subplot(2, 2, 4)
# ax.imshow(rgb2)
# ax.set_title(r"$\rho$=20", fontsize=10)
# ax.xaxis.set_visible(False)
# ax.yaxis.set_visible(False)

# plt.show()

