###################################################
#   This code is based on EE367 HW6, task 2
#   (Gordon Wetzstein, 10/2021)
###################################################

# import packages
import numpy as np
from numpy.fft import fft2, ifft2

from pypher.pypher import psf2otf
from pathlib import Path
from tqdm import tqdm
from itertools import product
import torch
import matplotlib.pyplot as plt
import cv2

def removal_hqs_tv(b, mask, lam, rho, num_iters, anisotropic_tv=True):
    
    nonsh_idx = np.where(mask == 0)
    nonsh_num = nonsh_idx[0].shape[0]
    sh_idx = np.where(mask > 0)
    sh_num = sh_idx[0].shape[0]
    
    mu_nonsh = np.sum(b[nonsh_idx]) / nonsh_num
    mu_sh = np.sum(b[sh_idx]) / sh_num
    mu = mu_nonsh - mu_sh
    sigma2 = np.sum((b[sh_idx] - mu)**2) / sh_num
    mu = -np.abs(mu)
    # lam *= sigma2

    # define function handle to compute horizontal and vertical gradients
    dx = np.array([[-1., 1.]])
    dy = np.array([[-1.], [1.]])
    dxFT = psf2otf(dx, b.shape)
    dyFT = psf2otf(dy, b.shape)
    dxTFT = np.conj(dxFT)
    dyTFT = np.conj(dyFT)
    dxyFT = np.stack((dxFT, dyFT), axis=0)
    dxyTFT = np.stack((dxTFT, dyTFT), axis=0)
    dI = np.array([[1.]])
    dIFT = psf2otf(dI, b.shape)
    
    
    grad_fn = lambda x: np.real(ifft2(fft2(x) * dxyFT))
    denom = rho * (np.abs(dxFT) ** 2 + np.abs(dyFT) ** 2) + dIFT
    
    # initialize x,z
    x = np.zeros_like(b)
    x[nonsh_idx] = b[nonsh_idx]
    z = grad_fn(x)

    for it in tqdm(range(num_iters)):        
        
        z1FT = fft2(z[0])
        z2FT = fft2(z[1])
        x_sh = ifft2((fft2(b-mu) + rho * (dxTFT * z1FT + dyTFT * z2FT)) / denom)
        x[nonsh_idx] = b[nonsh_idx]
        x[sh_idx] = np.real(x_sh)[sh_idx]

        # z update - soft shrinkage    
        # kappa = lam / rho
        kappa1 = 0
        kappa2 = 0
        v = grad_fn(x)

        # proximal operator of anisotropic TV term
        if anisotropic_tv:
            v0 = v[0,:,:]
            v1 = v[1,:,:]
            z0 = z[0,:,:]
            z1 = z[1,:,:]
            kappa1 = np.mean(np.abs(v0[sh_idx]))
            kappa2 = np.mean(np.abs(v1[sh_idx]))
            z0[sh_idx] = np.maximum(1 - kappa1/(np.abs(v0[sh_idx])), 0) * v0[sh_idx]
            z1[sh_idx] = np.maximum(1 - kappa2/(np.abs(v1[sh_idx])), 0) * v1[sh_idx]
            z[0,:,:] = z0
            z[1,:,:] = z1  
        
        # proximal operator of isotropic TV term
        else:
            v0 = v[0,:,:]
            v1 = v[1,:,:]
            vnorm = np.sqrt(v0[sh_idx]**2 + v1[sh_idx]**2 )
            z0 = z[0,:,:]
            z1 = z[1,:,:]
            kappa = np.mean(np.abs(vnorm))
            z0[sh_idx] = np.maximum(1 - kappa/vnorm,0) * v0[sh_idx]
            z1[sh_idx] = np.maximum(1 - kappa/vnorm,0) * v1[sh_idx] 
            z[0,:,:] = z0
            z[1,:,:] = z1  

    return x


